﻿using BankContracts.BindingModels;
using BankContracts.BusinessLogicsContracts;
using BankContracts.SearchModels;
using BankContracts.StoragesContracts;
using BankContracts.ViewModels;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BankBusinessLogic.BusinessLogics
{
    public class PaymentLogic : IPaymentLogic
    {
        private readonly ILogger _logger;
        private readonly IPaymentStorage _paymentStorage;

        public PaymentLogic(ILogger<PaymentLogic> logger, IPaymentStorage paymentStorage)
        {
            _logger = logger;
            _paymentStorage = paymentStorage;
        }

        public List<PaymentViewModel>? ReadList(PaymentSearchModel? model)
        {
            _logger.LogInformation("ReadList. Id:{Id}", model?.Id);
            var list = model == null ? _paymentStorage.GetFullList() : _paymentStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }

        public PaymentViewModel? ReadElement(PaymentSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement. Id:{Id}", model.Id);
            var element = _paymentStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
            return element;
        }

        public bool Create(PaymentBindingModel model)
        {
            CheckModel(model);
            if (_paymentStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }
        public bool Update(PaymentBindingModel model)
        {
            CheckModel(model);
            if (_paymentStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }

        public bool Delete(PaymentBindingModel model)
        {
            CheckModel(model);
            _logger.LogInformation("Delete. Id:{Id}", model.Id);
            if (_paymentStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }

        private void CheckModel(PaymentBindingModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("Payment. PaymentDate:{PaymentDate}. Id:{Id}", model.PaymentDate, model.Id);
        }
    }
}
